@@ -459,7 +459,7 @@ static off_t lseek_offset;
 int
 write_with_trace (int fd, void *varg, size_t len, char *file, int line)
 {
-  int  i;
+  int i = ARRAY_SIZE (rw_table) - 1;
   int ret;
   procfs_ctl_t *arg = (procfs_ctl_t *) varg;
 
@@ -747,7 +747,7 @@ procfs_note (char *msg, char *file, int line)
       if (info_verbose)
 	fprintf (procfs_file ? procfs_file : stdout, 
 		 "%s:%d -- ", file, line);
-      fprintf (procfs_file ? procfs_file : stdout, msg);
+      fprintf (procfs_file ? procfs_file : stdout, "%s", msg);
       if (procfs_file)
 	fflush (procfs_file);
     }
@@ -784,7 +784,7 @@ _initialize_proc_api (void)
 		   var_boolean, (char *) &procfs_trace, 
 		   "Set tracing for /proc api calls.\n", &setlist);
 
-  add_show_from_set (c, &showlist);
+  deprecated_add_show_from_set (c, &showlist);
   set_cmd_sfunc (c, set_procfs_trace_cmd);
   set_cmd_completer (c, filename_completer);
 
@@ -792,6 +792,6 @@ _initialize_proc_api (void)
 		   (char *) &procfs_filename, 
 		   "Set filename for /proc tracefile.\n", &setlist);
 
-  add_show_from_set (c, &showlist);
+  deprecated_add_show_from_set (c, &showlist);
   set_cmd_sfunc (c, set_procfs_file_cmd);
 }
