@@ -1,7 +1,7 @@
 /* Do various things to symbol tables (other than lookup), for GDB.
 
    Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
-   1995, 1996, 1997, 1998, 1999, 2000, 2002, 2003 Free Software
+   1995, 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004 Free Software
    Foundation, Inc.
 
    This file is part of GDB.
@@ -37,7 +37,7 @@
 #include "dictionary.h"
 
 #include "gdb_string.h"
-#include <readline/readline.h>
+#include "readline/readline.h"
 
 #ifndef DEV_TTY
 #define DEV_TTY "/dev/tty"
@@ -99,12 +99,12 @@ free_symtab_block (struct objfile *objfile, struct block *b)
 
   ALL_BLOCK_SYMBOLS (b, iter, sym)
     {
-      xmfree (objfile->md, DEPRECATED_SYMBOL_NAME (sym));
-      xmfree (objfile->md, sym);
+      xfree (DEPRECATED_SYMBOL_NAME (sym));
+      xfree (sym);
     }
 
   dict_free (BLOCK_DICT (b));
-  xmfree (objfile->md, b);
+  xfree (b);
 }
 
 /* Free all the storage associated with the struct symtab <- S.
@@ -116,10 +116,10 @@ free_symtab_block (struct objfile *objfile, struct block *b)
    It is s->free_code that says which alternative to use.  */
 
 void
-free_symtab (register struct symtab *s)
+free_symtab (struct symtab *s)
 {
-  register int i, n;
-  register struct blockvector *bv;
+  int i, n;
+  struct blockvector *bv;
 
   switch (s->free_code)
     {
@@ -138,7 +138,7 @@ free_symtab (register struct symtab *s)
       for (i = 0; i < n; i++)
 	free_symtab_block (s->objfile, BLOCKVECTOR_BLOCK (bv, i));
       /* Free the blockvector itself.  */
-      xmfree (s->objfile->md, bv);
+      xfree (bv);
       /* Also free the linetable.  */
 
     case free_linetable:
@@ -146,7 +146,7 @@ free_symtab (register struct symtab *s)
          or by some other symtab, except for our linetable.
          Free that now.  */
       if (LINETABLE (s))
-	xmfree (s->objfile->md, LINETABLE (s));
+	xfree (LINETABLE (s));
       break;
     }
 
@@ -156,12 +156,12 @@ free_symtab (register struct symtab *s)
 
   /* Free source-related stuff */
   if (s->line_charpos != NULL)
-    xmfree (s->objfile->md, s->line_charpos);
+    xfree (s->line_charpos);
   if (s->fullname != NULL)
-    xmfree (s->objfile->md, s->fullname);
+    xfree (s->fullname);
   if (s->debugformat != NULL)
-    xmfree (s->objfile->md, s->debugformat);
-  xmfree (s->objfile->md, s);
+    xfree (s->debugformat);
+  xfree (s);
 }
 
 void
@@ -230,16 +230,12 @@ print_objfile_statistics (void)
     if (OBJSTAT (objfile, sz_strtab) > 0)
       printf_filtered ("  Space used by a.out string tables: %d\n",
 		       OBJSTAT (objfile, sz_strtab));
-    printf_filtered ("  Total memory used for psymbol obstack: %d\n",
-		     obstack_memory_used (&objfile->psymbol_obstack));
+    printf_filtered ("  Total memory used for objfile obstack: %d\n",
+		     obstack_memory_used (&objfile->objfile_obstack));
     printf_filtered ("  Total memory used for psymbol cache: %d\n",
 		     bcache_memory_used (objfile->psymbol_cache));
     printf_filtered ("  Total memory used for macro cache: %d\n",
 		     bcache_memory_used (objfile->macro_cache));
-    printf_filtered ("  Total memory used for symbol obstack: %d\n",
-		     obstack_memory_used (&objfile->symbol_obstack));
-    printf_filtered ("  Total memory used for type obstack: %d\n",
-		     obstack_memory_used (&objfile->type_obstack));
   }
   immediate_quit--;
 }
@@ -579,7 +575,7 @@ Arguments missing: an output file name and an optional symbol file name");
 
   immediate_quit++;
   ALL_SYMTABS (objfile, s)
-    if (symname == NULL || (STREQ (symname, s->filename)))
+    if (symname == NULL || strcmp (symname, s->filename) == 0)
     dump_symtab (objfile, s, outfile);
   immediate_quit--;
   do_cleanups (cleanups);
@@ -818,7 +814,7 @@ maintenance_print_psymbols (char *args, int from_tty)
 
   immediate_quit++;
   ALL_PSYMTABS (objfile, ps)
-    if (symname == NULL || (STREQ (symname, ps->filename)))
+    if (symname == NULL || strcmp (symname, ps->filename) == 0)
     dump_psymtab (objfile, ps, outfile);
   immediate_quit--;
   do_cleanups (cleanups);
@@ -965,7 +961,7 @@ maintenance_print_msymbols (char *args, int from_tty)
 
   immediate_quit++;
   ALL_OBJFILES (objfile)
-    if (symname == NULL || (STREQ (symname, objfile->name)))
+    if (symname == NULL || strcmp (symname, objfile->name) == 0)
     dump_msymbols (objfile, outfile);
   immediate_quit--;
   fprintf_filtered (outfile, "\n\n");
@@ -1129,13 +1125,13 @@ maintenance_info_psymtabs (char *regexp, int from_tty)
 void
 maintenance_check_symtabs (char *ignore, int from_tty)
 {
-  register struct symbol *sym;
-  register struct partial_symbol **psym;
-  register struct symtab *s = NULL;
-  register struct partial_symtab *ps;
+  struct symbol *sym;
+  struct partial_symbol **psym;
+  struct symtab *s = NULL;
+  struct partial_symtab *ps;
   struct blockvector *bv;
-  register struct objfile *objfile;
-  register struct block *b;
+  struct objfile *objfile;
+  struct block *b;
   int length;
 
   ALL_PSYMTABS (objfile, ps)
@@ -1214,7 +1210,7 @@ maintenance_check_symtabs (char *ignore, int from_tty)
 static int
 block_depth (struct block *block)
 {
-  register int i = 0;
+  int i = 0;
   while ((block = BLOCK_SUPERBLOCK (block)) != NULL)
     {
       i++;
@@ -1228,7 +1224,7 @@ block_depth (struct block *block)
    be freed in free_objfile().  */
 
 void
-extend_psymbol_list (register struct psymbol_allocation_list *listp,
+extend_psymbol_list (struct psymbol_allocation_list *listp,
 		     struct objfile *objfile)
 {
   int new_size;
@@ -1236,14 +1232,14 @@ extend_psymbol_list (register struct psymbol_allocation_list *listp,
     {
       new_size = 255;
       listp->list = (struct partial_symbol **)
-	xmmalloc (objfile->md, new_size * sizeof (struct partial_symbol *));
+	xmalloc (new_size * sizeof (struct partial_symbol *));
     }
   else
     {
       new_size = listp->size * 2;
       listp->list = (struct partial_symbol **)
-	xmrealloc (objfile->md, (char *) listp->list,
-		   new_size * sizeof (struct partial_symbol *));
+	xrealloc ((char *) listp->list,
+		  new_size * sizeof (struct partial_symbol *));
     }
   /* Next assumes we only went one over.  Should be good if
      program works correctly */
